package jshow.detail;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.image.BufferedImage;

import javax.swing.JPanel;

import jshow.detail.filter.BlurFilteredImage;
import jshow.detail.filter.ContrastFilteredImage;
import jshow.detail.filter.GrayFilteredImage;
import jshow.detail.filter.GrayscaleFilteredImage;
import jshow.detail.filter.MarbleFilteredImage;
import jshow.detail.filter.NoiseFilteredImage;
import jshow.detail.filter.OriginalImage;
import jshow.detail.filter.WeaveFilteredImage;
import jshow.util.ImageUtil;

public class FilteredImagePanel extends JPanel {

	private static final int HEIGHT = 95;

	private static final int WIDTH = 50;

	public static final int IMAGE_HEIGHT = 85;

	public static final int IMAGE_WIDTH = 85;

	public static final int MARGIN_WIDTH = 10;

	public static final int MARGIN_HEIGHT = 5;

	private BufferedImage image = null;

	private BufferedImage originalImage = null;

	private JPanel parentPanel = null;

	public FilteredImagePanel(JPanel parentPanel, BufferedImage originalimage) {
		super();
		this.setBackground(Color.WHITE);
		this.setPreferredSize(new Dimension(WIDTH, HEIGHT));
		this.setLayout(new FlowLayout(FlowLayout.CENTER, MARGIN_WIDTH,
				MARGIN_HEIGHT));
		this.image = ImageUtil.getScaledImage(originalimage, IMAGE_HEIGHT,
				IMAGE_WIDTH);
		this.parentPanel = parentPanel;
		this.originalImage = originalimage;
		makeImage();
	}

	private void makeImage() {
		add(new OriginalImage(parentPanel, image, originalImage));
		add(new BlurFilteredImage(parentPanel, image, originalImage));
		add(new MarbleFilteredImage(parentPanel, image, originalImage));
		add(new ContrastFilteredImage(parentPanel, image, originalImage));
		add(new NoiseFilteredImage(parentPanel, image, originalImage));
		add(new GrayFilteredImage(parentPanel, image, originalImage));
		add(new GrayscaleFilteredImage(parentPanel, image, originalImage));
		add(new WeaveFilteredImage(parentPanel, image, originalImage));
	}
}
