package jshow.detail.filter;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.FilteredImageSource;
import java.awt.image.ImageFilter;
import java.awt.image.ImageProducer;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.LineBorder;

import jshow.detail.FilteredImagePanel;
import jshow.util.ImageUtil;

public abstract class FilteredImage extends JButton implements ActionListener {

	private Image image = null;

	private Image originalImage = null;

	private JPanel parentPanel = null;

	ImageFilter imageFilter = null;

	public FilteredImage(JPanel parentPanel, Image image, Image originalImage) {

		this.parentPanel = parentPanel;
		this.image = image;
		this.originalImage = originalImage;
		this.setBackground(Color.WHITE);
		this.setPreferredSize(new Dimension(FilteredImagePanel.IMAGE_HEIGHT,
				FilteredImagePanel.IMAGE_HEIGHT));
		this.setBorder(new LineBorder(Color.ORANGE, 1));
		this.addActionListener(this);
		setImageFilter();

		setIcon(new ImageIcon(ImageUtil
				.toBufferedImage(createImage(getImageProducer()))));
	}

	public void actionPerformed(ActionEvent event) {
		JLabel label = null;

		parentPanel.removeAll();

		label = new JLabel(new ImageIcon(createImage(getFullImageProducer())));

		label.setBackground(Color.WHITE);
		parentPanel.add(label, BorderLayout.CENTER);

		parentPanel.revalidate();
		parentPanel.repaint();
	}

	abstract void setImageFilter();

	public ImageProducer getImageProducer() {
		if (imageFilter == null)
			return image.getSource();

		return new FilteredImageSource(image.getSource(), imageFilter);
	}

	public ImageProducer getFullImageProducer() {
		if (imageFilter == null)
			return originalImage.getSource();

		return new FilteredImageSource(originalImage.getSource(), imageFilter);
	}
}
