package jshow.thumb;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;

import jshow.util.ImageUtil;

public class RunnableImage implements Runnable {

	private JPanel panel;

	private JComponent parentPanel;

	private File file;

	private ActionListener listener;

	private JScrollPane scrollPane;

	public RunnableImage(JComponent parentPanel, JScrollPane scrollPane,
			ActionListener listener, File file) {
		panel = new JPanel();
		panel.setLayout(new BorderLayout());
		this.file = file;
		this.parentPanel = parentPanel;
		this.scrollPane = scrollPane;
		this.listener = listener;
	}

	public void run() {
		BufferedImage temp = null;
		ImageButton button = null;

		try {
			temp = ImageIO.read(file);

			button = new ImageButton(file, new ImageIcon(ImageUtil
					.getScaledImage(temp, ThumbPanel.WIDTH, ThumbPanel.HEIGHT)));
			button.addActionListener(listener);

			button.setBackground(Color.WHITE);

			panel.add(button, BorderLayout.CENTER);

			panel.setPreferredSize(new Dimension(ThumbPanel.WIDTH,
					ThumbPanel.HEIGHT));

			SwingUtilities.invokeAndWait(new Runnable() {
				public void run() {
					parentPanel.add(panel);
					scrollPane.setViewportView(parentPanel);
				}
			});

			scrollPane.revalidate();
			scrollPane.repaint();

		} catch (Exception e) {
			System.out.println("File processing Error : "
					+ file.getAbsolutePath());
			e.printStackTrace();
		} finally {
			if (temp != null)
				temp.flush();
		}
	}
}
