package jshow.thumb;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.io.File;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.imageio.ImageIO;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.ScrollPaneConstants;

import jshow.detail.EditImagePanel;

public class ThumbPanel extends JTabbedPane {

	private JPanel thPanel = null;

	private JScrollPane scrollPane = null;

	private EditImagePanel editPane = null;

	public static final int WIDTH = 150;

	public static final int HEIGHT = 110;

	public static final int CORE_THREAD = 5;

	public static final int MAX_THREAD = 10;

	public static final int KEEP_ALIVE_TIME = 1;

	public static final int MAX_WAIT_TIME = 1;

	public static final int MARGIN_WIDTH = 10;

	public static final int MARGIN_HEIGHT = 10;

	private static final int MARGIN = 8;

	private File[] imageFiles = null;

	private Runnable lastJob = null;

	private ThreadPoolExecutor threadPool;

	private LinkedBlockingQueue<Runnable> queue;

	class ResizeListener extends ComponentAdapter {
		public void componentResized(ComponentEvent e) {
			resizePanel();
		}
	}

	class ChangePanelListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			remove(editPane);
			editPane = new EditImagePanel(((ImageButton) event.getSource())
					.getFile());
			addTab("", null, editPane, "img");
			setSelectedIndex(1);
			setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
	}

	private void resizePanel() {
		double area = 0;

		if (imageFiles != null && imageFiles.length > 0) {
			area = imageFiles.length * (WIDTH + MARGIN_WIDTH * 2)
					* (HEIGHT + MARGIN_HEIGHT * 2) + MARGIN
					* (WIDTH + MARGIN_WIDTH * 2) * (HEIGHT + MARGIN_HEIGHT * 2);
		}

		if (thPanel.getVisibleRect().width > 0) {
			thPanel.setPreferredSize(new Dimension(
					thPanel.getVisibleRect().width, (int) Math
							.ceil((area / thPanel.getVisibleRect().width)
									/ HEIGHT)
							* HEIGHT));
		}

		scrollPane.setViewportView(thPanel);
	}

	public ThumbPanel() {
		queue = new LinkedBlockingQueue<Runnable>();
		threadPool = new ThreadPoolExecutor(CORE_THREAD, MAX_THREAD,
				KEEP_ALIVE_TIME, TimeUnit.SECONDS, queue);

		ImageIO.setUseCache(true);
		thPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, MARGIN_WIDTH,
				MARGIN_HEIGHT));
		scrollPane = new JScrollPane(
				ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPane.addComponentListener(new ResizeListener());

		this.addTab("׸", null, scrollPane, "tile");
		scrollPane.setViewportView(thPanel);
	}

	public void setImageFiles(File[] imageFiles) {

		stopLoading();

		this.imageFiles = imageFiles;

		resizePanel();

		thPanel.removeAll();

		for (int i = 0; i < imageFiles.length; i++) {
			threadPool.execute(new RunnableImage(thPanel, scrollPane,
					new ChangePanelListener(), imageFiles[i]));
		}

		if (lastJob != null) {
			threadPool.execute(lastJob);
		}
	}

	public void stopLoading() {
		if (threadPool != null) {
			threadPool.purge();
		}
	}

	public void setLastJob(Runnable r) {
		lastJob = r;
	}
}
